@@ -137,19 +137,13 @@ static WIN_BOOL MODULE_InitDll( WINE_MODREF *wm, DWORD type, LPVOID lpReserved )
 #endif
     assert( wm );
 
-
     /* Skip calls for modules loaded with special load flags */
 
     if (    ( wm->flags & WINE_MODREF_DONT_RESOLVE_REFS )
          || ( wm->flags & WINE_MODREF_LOAD_AS_DATAFILE ) )
         return TRUE;
 
-
-#ifdef LOG
     TRACE("(%s,%s,%p) - CALL\n", wm->modname, typeName[type], lpReserved );
-#else
-    TRACE("(%s,%p) - CALL\n", wm->modname, lpReserved );
-#endif
 
     /* Call the initialization routine */
     switch ( wm->type )
@@ -171,11 +165,7 @@ static WIN_BOOL MODULE_InitDll( WINE_MODREF *wm, DWORD type, LPVOID lpReserved )
     /* The state of the module list may have changed due to the call
        to PE_InitDLL. We cannot assume that this module has not been
        deleted.  */
-#ifdef LOG
     TRACE("(%p,%s,%p) - RETURN %d\n", wm, typeName[type], lpReserved, retv );
-#else
-    TRACE("(%p,%p) - RETURN %d\n", wm, lpReserved, retv );
-#endif
 
     return retv;
 }
@@ -360,15 +350,12 @@ static WIN_BOOL MODULE_FreeLibrary( WINE_MODREF *wm )
 HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 {
 	WINE_MODREF *wm = 0;
-	char* listpath[] = { "", "", WIN32_PATH, "/usr/local/lib/win32", WIN32_LIB_PATH, NULL };
+	char* listpath[] = { "", "", WIN32_LIB_PATH, WIN32_PATH, "/usr/local/lib/win32", NULL };
 	char path[512];
-	char checked[2000];
 	int i;
     static char *unavailable_lib_names[100];
     static int unavailable_libs=0;
-    
 
-	checked[0] = 0;
 	if(!libname)
 	{
 		SetLastError(ERROR_INVALID_PARAMETER);
@@ -388,7 +375,7 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 
     //if(fs_installed==0)
     //    install_fs();
-    
+
 	TRACE("module '%s' not already loaded\n", libname);
 
     i = -1;
@@ -399,12 +386,12 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 
 	    if (i < 2)
 	    {
-          if (i == 0)
-              /* check just original file name */
-              strncpy(path, libname, sizeof(path) - 1);
-          else
-              /* check default user path */
-              strncpy(path, win32_def_path, sizeof(path) - 2);
+            if (i == 0)
+                /* check just original file name */
+                strncpy(path, libname, sizeof(path) - 1);
+            else
+                /* check default user path */
+                strncpy(path, win32_def_path, sizeof(path) - 2);
 	    }
 	    else if (strcmp(win32_def_path, listpath[i]))
             /* path from the list */
@@ -419,27 +406,24 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 	    }
 	    path[511] = 0;
 
-		//TRACE("trying to load module '%s'\n", path);
+		TRACE("trying to load module '%s'\n", path);
 	    wm = MODULE_LoadLibraryExA( path, hfile, flags );
 
 	    if (!wm)
 	    {
-            if (checked[0])
-                strcat(checked, ", ");
-            strcat(checked, path);
-            checked[1999] = 0;
+	        TRACE("loading module '%s' failed\n", path);
 	    }
 	}
 	if ( wm )
 	{
-		//TRACE("Loaded module '%s'\n", libname);
+		TRACE("Loaded module '%s'\n", libname);
 
 		if ( !MODULE_DllProcessAttach( wm, NULL ) )
 		{
 			WARN_(module)("Attach failed for module '%s', \n", libname);
+			MODULE_RemoveFromList(wm);
 			MODULE_FreeLibrary(wm);
 			SetLastError(ERROR_DLL_INIT_FAILED);
-			MODULE_RemoveFromList(wm);
 			wm = NULL;
 		}
 	}
@@ -583,6 +567,8 @@ WIN_BOOL WINAPI FreeLibrary(HINSTANCE hLibModule)
 
     wm=MODULE32_LookupHMODULE(hLibModule);
 
+    MODULE_RemoveFromList(wm);
+
     if ( !wm || !hLibModule )
     {
         SetLastError( ERROR_INVALID_HANDLE );
@@ -591,8 +577,6 @@ WIN_BOOL WINAPI FreeLibrary(HINSTANCE hLibModule)
     else
         retv = MODULE_FreeLibrary( wm );
 
-    MODULE_RemoveFromList(wm);
-
     /* garbage... */
     if (local_wm == NULL) my_garbagecollection();
 
@@ -1054,12 +1038,14 @@ void CodecRelease(void)
     {
 	for (;;)
 	{
+	    WINE_MODREF* wm;
 	    modref_list* list = local_wm;
 	    if (!local_wm)
 		break;
 	    //printf("CODECRELEASE %p\n", list);
-            MODULE_FreeLibrary(list->wm);
-	    MODULE_RemoveFromList(list->wm);
+	    wm=list->wm;
+	    MODULE_RemoveFromList(wm);
+            MODULE_FreeLibrary(wm);
             if (local_wm == NULL)
 		my_garbagecollection();
 	}
