@@ -1,4 +1,4 @@
-/* $Id: BuzzMachineLoader.cpp 2457 2009-06-03 21:58:45Z ensonic $
+/* $Id: BuzzMachineLoader.cpp 3017 2010-09-24 18:20:51Z ensonic $
  *
  * Buzz Machine Loader
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -338,7 +338,7 @@ extern "C" DE void bm_free(BuzzMachine *bm) {
             else {
                 delete (BuzzMachineCallbacks *)callbacks;
                 DBG("  freed callback instance\n");
-            }         
+            }
         }
         free(bm);
     }
@@ -371,6 +371,7 @@ extern "C" DE BuzzMachine *bm_new(BuzzMachineHandle *bmh) {
       DBG("  callback instance created\n");
     }
 
+    // FIXME: should we do this earlier?
     bm->machine_iface->pMasterInfo=&master_info;
     bm->machine_iface->pCB=bm->callbacks;
 
@@ -424,6 +425,9 @@ extern "C" DE void bm_init(BuzzMachine *bm, unsigned long blob_size, unsigned ch
 	bm->machine_iface->SetNumTracks(bm->machine_info->minTracks);
     DBG1("  CMachineInterface::SetNumTracks(%d) called\n",bm->machine_info->minTracks);
 
+    // FIXME: CyanPhase DTMF-1 access gval.xxx in mi::Init
+    // -> so we might need to call these before
+
     // initialise global parameters (DefValue or NoValue, Buzz seems to use NoValue)
     for(i=0;i<bm->machine_info->numGlobalParameters;i++) {
         if(bm->machine_info->Parameters[i]->Flags&MPF_STATE) {
@@ -474,7 +478,9 @@ extern "C" DE void * bm_get_track_parameter_location(BuzzMachine *bm,int track,i
     if(!(bm->machine_iface->TrackVals)) return(0);
 
     byte *ptr=(byte *)bm->machine_iface->TrackVals;
-    if(!ptr) DBG(" -> machine->TrackVals is NULL!\n");
+    if(!ptr) {
+      DBG(" -> machine->TrackVals is NULL!\n");
+    }
 
     // @todo prepare pointer array in bm_init
     for(int j=0;j<=track;j++) {
@@ -539,7 +545,9 @@ extern "C" DE void * bm_get_global_parameter_location(BuzzMachine *bm,int index)
     if(!(bm->machine_iface->GlobalVals)) return(0);
 
     byte *ptr=(byte *)bm->machine_iface->GlobalVals;
-    if(!ptr) DBG(" -> machine->GlobalVals is NULL!\n");
+    if(!ptr) {
+      DBG(" -> machine->GlobalVals is NULL!\n");
+    }
 
     // @todo prepare pointer array in bm_init
     for(int i=0;i<=index;i++) {
